"""Create a development and testing environment using a virtualenv."""

from __future__ import unicode_literals
import os
import subprocess
import sys


if sys.version_info[0] >= 3:
    VENV_NAME = '.venv3'
    # FIXME: running 2to3 on django-nose will no longer be required once
    # the project supports Python 3 (bug #16).
    PATCH_DJANGO_NOSE = True
else:
    VENV_NAME = '.venv'
    PATCH_DJANGO_NOSE = False


TESTS = os.path.abspath(os.path.dirname(__file__))
REQUIREMENTS = os.path.join(TESTS, 'requirements.pip')
WITH_VENV = os.path.join(TESTS, 'with_venv.sh')
VENV = os.path.abspath(os.path.join(TESTS, '..', VENV_NAME))


def call(*args):
    """Simple ``subprocess.call`` wrapper."""
    if subprocess.call(args):
        raise SystemExit('Error running {0}.'.format(args))


def pip_install(*args):
    """Install packages using pip inside the virtualenv."""
    call(WITH_VENV, VENV_NAME, 'pip', 'install', '--use-mirrors', *args)


def patch_django_nose():
    """Run 2to3 on django-nose and remove ``import new`` from its runner."""
    # FIXME: delete once django-nose supports Python 3 (bug #16).
    python = 'python' + '.'.join(map(str, sys.version_info[:2]))
    django_nose = os.path.join(
        VENV, 'lib', python, 'site-packages', 'django_nose')
    call('2to3', '-w', '--no-diffs', django_nose)
    with open(os.path.join(django_nose, 'runner.py'), 'r+') as f:
        lines = [line for line in f.readlines() if 'import new' not in line]
        f.seek(0)
        f.truncate()
        f.writelines(lines)


if __name__ == '__main__':
    call('virtualenv', '--distribute', '-p', sys.executable, VENV)
    pip_install('-r', REQUIREMENTS)
    # FIXME: delete from now on once django-nose supports Python 3  (bug #16).
    if PATCH_DJANGO_NOSE:
        patch_django_nose()



from django import template
from django.contrib.sites.models import Site
from django.utils.safestring import mark_safe

register = template.Library()

from amp_tools.settings import settings

@register.simple_tag
def amp_canonical_link(request):
    getvars = request.GET.copy()
    rel = "amphtml"
    if settings.AMP_TOOLS_GET_PARAMETER in getvars:
        del getvars[settings.AMP_TOOLS_GET_PARAMETER]
        rel = "canonical"
    else:
        getvars[settings.AMP_TOOLS_GET_PARAMETER] = settings.AMP_TOOLS_GET_VALUE

    if len(getvars.keys()) > 0:
        getvars = getvars.urlencode()
    else:
        getvars = ''

    href = '%s?%s' % ( request.path, getvars) if getvars else request.path

    href = "%s://%s%s" % (request.scheme, Site.objects.get_current().domain, href)

    return mark_safe('<link rel="%s" href="%s" />' % (rel, href) )




from amp_tools.settings import settings
from amp_tools import set_amp_detect


class AMPDetectionMiddleware(object):

    def process_request(self, request):
        if settings.AMP_TOOLS_GET_PARAMETER in request.GET:
            if request.GET[settings.AMP_TOOLS_GET_PARAMETER] == settings.AMP_TOOLS_GET_VALUE:
                set_amp_detect(is_amp_detect=True, request=request)




import threading

from django.test import TestCase

from mock import MagicMock, Mock, patch, call

from amp_tools import get_amp_detect
from amp_tools.middleware import AMPDetectionMiddleware

def _reset():
    '''
    Reset the thread local.
    '''
    import amp_tools
    del amp_tools._local
    amp_tools._local = threading.local()


class BaseTestCase(TestCase):
    def setUp(self):
        _reset()

    def tearDown(self):
        _reset()


class DetectAMPMiddlewareTests(BaseTestCase):

    def test_default_page(self):
        request = Mock()
        request.META = MagicMock()
        request.GET = {}
        middleware = AMPDetectionMiddleware()
        middleware.process_request(request)
        # default flavour is set
        self.assertEqual(get_amp_detect(), '')

    @patch('amp_tools.middleware.set_amp_detect')
    def test_set_amp_detect_through_get_parameter(self, set_amp_detect):
        request = Mock()
        request.META = MagicMock()
        request.GET = {'amp-content': 'amp'}
        middleware = AMPDetectionMiddleware()
        middleware.process_request(request)
        self.assertEqual(set_amp_detect.call_args, call(is_amp_detect=True, request=request))



import os

PROJECT_ROOT = os.path.dirname(os.path.abspath(__file__))
join_to_project = lambda slug: os.path.join(PROJECT_ROOT, slug)

SECRET_KEY = 'amp-tools-key'

DATABASES = {'default': {'ENGINE': 'django.db.backends.sqlite3'}}

INSTALLED_APPS = [
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.sites',
    'django.contrib.admin',
    'amp_tools',

]

TEMPLATE_CONTEXT_PROCESSORS = [
    "django.contrib.auth.context_processors.auth",
    "django.core.context_processors.debug",
    "django.core.context_processors.i18n",
    "django.core.context_processors.media",
]

TEMPLATE_LOADERS = [
    'amp_tools.loader.Loader',
    'django.template.loaders.filesystem.Loader',
    'django.template.loaders.app_directories.Loader',
]


TEMPLATES = [{
    'BACKEND': 'django.template.backends.django.DjangoTemplates',
    'DIRS': [
        # insert your TEMPLATE_DIRS here
        join_to_project('templates'),
    ],
    # 'APP_DIRS': True,
    'OPTIONS': {
        'context_processors': TEMPLATE_CONTEXT_PROCESSORS,
        'loaders': TEMPLATE_LOADERS,
    },
}]


MIDDLEWARE_CLASSES = (
    'django.middleware.common.CommonMiddleware',
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'amp_tools.middleware.AMPDetectionMiddleware',
)


TEMPLATE_DIRS = (
    os.path.join(PROJECT_ROOT, 'templates'),
)

TEST_RUNNER = 'django.test.runner.DiscoverRunner'
RUNTESTS_DIR = os.path.abspath(os.path.dirname(__file__))
